#!/usr/bin/perl

use strict;

my $kavconfig='/etc/kav/5.0/kav4unix.conf';
my $hostname=`hostname`;
my $smtp_bin;
my $smtp_config;
my $license_installed = 0;
my $lic_path = "/var/db/kav/5.0/kav4unix/keys/";

sub is_dir_exist{
  my $dir = shift;
  if (-d $dir) {
    return 0;
  }
  return 1;
}
my $top_dir;

$top_dir='/opt/kav' if $^O=~/linux/i;
$top_dir='/usr/local/share/kav' if $^O=~/bsd/i;

my $path_bin=$top_dir.'/bin';


sub dir_remove_trailing_slashes {
  my $path = shift;
  for(;;) {
    my $len;
    my $pos;
    $len = length($path);
    if ($len < 2) {
    # Could be '/' or any other character. Ok.
      return $path;
    }
    $pos = rindex($path, '/');
    if ($pos != $len - 1) {
    # No trailing slash
      return $path;
    }
    # Remove the trailing slash
    $path = substr($path, 0, $len - 1)
  }
}

sub internal_dirname {
  my $path = shift;
  my $pos;

  $path = dir_remove_trailing_slashes($path);

  $pos = rindex($path, '/');
  if ($pos == -1) {
    # No slash
    return '.';
  }

  if ($pos == 0) {
    # The only slash is at the beginning
    return '/';
  }

  return substr($path, 0, $pos);
}

sub query {
  my $message = shift;
  my $default = shift;
  my $answer;
  print ($message . (($default eq '') ? '' : (' [' . $default . ']:')));
  chomp($answer = <STDIN>);
  if ($answer eq '') {
    return $default;  
  }    
    return $answer;
}

sub get_answer {
  my $message = shift;
  my $default = shift;
  my $type = shift;
  my $var = shift;
  my $answer;
  $answer = query($message,$default);
  return $answer;
}

sub webmin_module_install {
  my $path_script;
  my $webmin_conf="/etc/webmin"; 
  while (is_dir_exist($webmin_conf)) {
    $webmin_conf=get_answer("\nWebmin configuration file is not found.\n"
    	. "(probably webmin is not installed, or installed into non-default location.)\n\n"
    	. "Webmin (www.webmin.com) is a web-based interface for system administration\n"
    	. "for various Unix utilites. If you install it, you'll be able to configure\n"
    	. "and use Kaspersky Anti-Virus through the web interface.\n"
    	. "If you want to use this functionality, but haven't installed webmin yet,\n"
    	. "you can safely type 'cancel' here, install webmin later and re-run this\n"
    	. "configuration script again (or use Webmin to install the module from directory\n"
    	. "$top_dir/contrib/kavfs.wbm)\n"
    	. "If you have webmin installed in non-default path, please enter\n"
    	. "where webmin configuration file located?\n(type \"cancel\" for continue installation without webmin).",
	'cancel') ;
    
    if($webmin_conf eq 'cancel') {
      return 1;
    }
  }
  unless ($webmin_conf eq 'cancel') {
    open(WBM,"<$webmin_conf/miniserv.conf") or
     (warn "Could not open config webmin file '$webmin_conf/miniserv.conf'. Webmin module installation aborted."
      and return 1 );
    while(<WBM>) {
      $path_script=$1 if (/^root=(.*)$/);
    }       
    close WBM;
    `perl $path_script/install-module.pl $top_dir/contrib/kavfs.wbm $webmin_conf`;
    print "Webmin module installed successfully. Check it on 'Others' web page.\n"
    	. "Depend on Webmin configuration, you may need to enable Kaspersky Anti-Virus\n"
    	. "Webmin module. To do this, on Webmin main page go to\n" 
    	. "Webmin -> Webmin users -> <username> -> check box on 'KAV for Unix'\n";
  }
}


sub error {
  my $msg = shift;

  print STDERR $msg." Execution aborted."."\n\n" ;
  exit 1;
}


sub safe_chmod {
  my $mode = shift;
  my $file = shift;

  if (chmod($mode, $file) != 1) {
    error('Unable to change the access rights of the file ' . $file . '.' . "\n\n");
  }
}

sub safe_mkdir {
  my $file = shift;

  if (mkdir($file, 0000) == 0) {
    error('Unable to create the directory ' . $file . '.' . "\n\n");
  }
}

sub create_dir {
  my $dir = shift;

  if (-d $dir) {
#    print "Directory $dir exists\n";
    return 1;
  }

  if (index($dir, '/') != -1) {
    create_dir(internal_dirname($dir));
  }
  safe_mkdir($dir);
  safe_chmod(0755, $dir);
  return 0;
}


sub get_key {
  my $dir = shift ;
  my @fn ;
  opendir(LS,$dir) ;
  @fn = grep(s/(.+(\.key))$/$dir\/$1/, readdir(LS));
  closedir(LS);
  return @fn;
} 

sub internal_basename {
  return substr($_[0], rindex($_[0], '/') + 1);
}
 
sub key_install {
  my $license_path = shift;
  print "\n";
  create_dir($license_path);
  if (my @keys=get_key("./")) {
  	print "Installing licenses from current directory.\n";
    foreach (@keys) {
      unless (-f $license_path."/".internal_basename($_)) {
          `cp $_ $license_path`;
      }
    }
    $license_installed = 1;
  }
  else {
    while(1) {
      my $dir=get_answer ("Kaspersky Anti-Virus requires license keys (*.key) to work.\n"
      . "If you don't have any license file, read the documentation how to get them.\n"
      . "Please enter directory which contains Kaspersky Lab license key files\n"
      . "(type \"cancel\" for continue installation without license installation.)",'cancel');
      if ($dir =~ /^cancel$/i) {
        return 1;
	last;
      }
      if (my @keys=get_key($dir)) {
        foreach (@keys) {
          unless (-f $license_path."/".internal_basename($_)) {
            `cp $_ $license_path`;
          }
          else {
            print "You already have installed the license file: ".internal_basename($_)."\n";
          }
		  $license_installed = 1;
        }
        last;
      }
      else {
        print "License file not found.\n";
      }
    }
  }    
}

############## main ###################

# For Webmin

&webmin_module_install();

#For license
my $license;
open (KAVCONF,"+< $kavconfig") or die "Could not open config file $kavconfig: $!\n";
  my @KAVCONFIG=<KAVCONF> ;
  foreach my $param (@KAVCONFIG) {
    if ($param=~/^LicensePath=(.*)/) {
         $lic_path=$1; 
      }
  }
close KAVCONF or die "Closing :$!\n";

&key_install($lic_path);

#For convert old setting
if ($^O=~/bsd/i) {
  if (-f '/usr/local/share/AVP/AvpUnix.ini' ) {
	print "Old Kaspersky Anti-Virus installation is found.\n";
	if ( get_answer ("Do you want to convert old settings?",'yes') eq 'yes')
	{
		print "Configuration file based on old settings will be stored in file /etc/kav/5.0/kav4unix.conf_4.0\n";
		`perl /usr/local/share/kav/contrib/converter.pl > /etc/kav/5.0/kav4unix.conf_4.0` ;
    }   
  }
}

if ($^O=~/linux/i) {
   if (-f '/opt/AVP/AvpUnix.ini') {
	print "Old Kaspersky Anti-Virus installation is found.\n";
	if ( get_answer ("Do you want to convert old settings?",'yes') eq 'yes')
	{
		print "Configuration file based on old settings will be stored in file /etc/kav/5.0/kav4unix.conf_4.0\n";
		`perl /opt/kav/contrib/converter.pl > /etc/kav/5.0/kav4unix.conf_4.0` ;
    }   
  }
}


print "******************************************************************************\n"
	. "Kaspersky Anti-Virus for ".uc($^O)." configuration completed.\n"
	. "Binaries were installed in $path_bin directory\n"
	. "Configuration file is $kavconfig\n"
	. "We recommend to add this directory in your PATH environment variable.\n"
	. "To access the man pages add the $top_dir/man path to the MANPATH env variable\n"
	. "To start using Kaspersky Anti-Virus you should download latest AV bases;\n"
	. "to do this run $path_bin/keepup2date.\n"
	. "Also if you're behind the proxy and want to update from Internet, you should\n"
	. "set environment variables http_proxy and/or ftp_proxy (see wget manual pages)\n";

if ( !$license_installed )
{
	print "\nWARNING: You didn't install any license. Before using the product, you must\n"
		. "install valid licenses (just copy them to the directory specified in\n"
		. "LicensePath - currently '$lic_path').\n"
		. "If you need to obtain the license, see the product documentation.\n"
}	
