#!/usr/bin/perl
use strict;
use IPC::Open2;
my $line;
my %sections;
my $sec;
my $oldconfig;
my $newconfig=$2 || 'STDOUT';

my $inifile;
my $kldbedit;
my $db_name;
my $users_db_name;

if ($^O=~/linux/i) {
  $inifile=shift || '/opt/AVP/AvpUnix.ini';
  $kldbedit="/opt/AVP/Tools/kldbedit";
  $db_name="/opt/AVP/etc/defUnix";
  $users_db_name="/opt/AVP/etc/userdb";
}
if ($^O=~/bsd/i) {
  $inifile=shift || '/usr/local/share/AVP/etc/AvpUnix.ini';
  $kldbedit="/usr/local/share/AVP/Tools/kldbedit";
  $db_name="/usr/local/share/AVP/etc/defUnix";
  $users_db_name="/usr/local/share/AVP/etc/userdb";
}

my $users_file="./users";
my $template_path="/tmp/"; 
my $mod_name = "smtpscan";

my %keeper_sections;

sub dir_remove_trailing_slashes {
  my $path = shift;
  for(;;) {
    my $len;
    my $pos;
    $len = length($path);
    if ($len < 2) {
    # Could be '/' or any other character. Ok.
      return $path;
    }
    $pos = rindex($path, '/');
    if ($pos != $len - 1) {
    # No trailing slash
      return $path;
    }
    # Remove the trailing slash
    $path = substr($path, 0, $len - 1)
  }
}

sub internal_dirname {
  my $path = shift;
  my $pos;

  $path = dir_remove_trailing_slashes($path);

  $pos = rindex($path, '/');
  if ($pos == -1) {
    # No slash
    return '.';
  }

  if ($pos == 0) {
    # The only slash is at the beginning
    return '/';
  }

  return substr($path, 0, $pos);
}
	
sub config_dumb{

open (INI,"< $inifile") or die "Could not open file $inifile: $!";
while($line=<INI>)
	{
		if ($line=~/^\[(.*)\]$/)
		{
			$sec=$1;
		}
		elsif ($line=~/^(.*)\=(.*)/)
		{
			$sections{$sec}{$1}=$2 ;
		}
	}
$oldconfig=$sections{'AVP32'}{'DefaultProfile'};	
unless ($oldconfig=~/^\/.*/)
	{
		my $dir=internal_dirname($inifile);
		$oldconfig=~ s/^(\.\/)(.*)/$2/;
		$oldconfig="$dir/$oldconfig";
	
	}
open(OLDCONFIG,"$oldconfig") or die "Could not open old configuraton file: $!";
while($line=<OLDCONFIG>)
	{
		if ($line=~/^\[(.*)\]$/)
		{
			$sec=$1;
		}
		elsif ($line=~/(.*)\=(.*)/)
		{
			$sections{$sec}{$1}=$2;
		}
	}		
	
close(OLDCONFIG); 	
}

config_dumb;

sub change_macro{
	my( $str) = @_;
	$str=~s/\%KAVANSWER\%/\%VIRUSNAME\%/m;
	$str=~s/\%RECIPIENT\%/\%RECIPIENTS\%/m;
	return $str;
}

sub get_setting{
	my( $db_name, $name) = @_;
	my $cmd = $kldbedit." --vc=".$name." ".$db_name;
	my $value = "";
	open2(*OUTPUT, *INPUT, $cmd);
	my $first_line = <OUTPUT>;
	if ($first_line=~/^[ \t]*?Name: .*? Value: (.*?)( | Subtopics:.*)$/)
	{
		$value=$1;
	}
	else
	{
		print("Could not get value \"$name\" from db \"$db_name\".\n");
	}
	close(INPUT);
	close(OUTPUT);
	return $value;
}
sub get_mlsetting{
	my( $db_name, $name) = @_;
	my $cmd = $kldbedit." --vc=".$name." ".$db_name;
	my $value = "";
	open2(*OUTPUT, *INPUT, $cmd);
	my $first_line = <OUTPUT>;
	if ($first_line=~/^[ \t]*?Name: .*? Value: (.*?)(| | Subtopics:.*)$/)
	{
		$value=$1;
		while($first_line=<OUTPUT>){
			$first_line =~s/Subtopics://;
			$value = $value.$first_line;
		}
	}
	else
	{
		print("Could not get value \"$name\" from db \"$db_name\".\n");
	}
	close(INPUT);
	close(OUTPUT);
	return $value;
}

sub get_bool_setting{
	my( $db_name, $name) = @_;
	my $value = get_setting($db_name, $name);
	if($value eq "1"){
		$value="yes";
	}else{
		$value="no";
	}
	return $value;
}

sub get_notbool_setting{
	my( $db_name, $name) = @_;
	my $value = get_setting($db_name, $name);
	if($value eq "1"){
		$value="no";
	}else{
		$value="yes";
	}
	return $value;
}

sub get_action_setting{
	my( $db_name, $name) = @_;
	my $value = get_setting($db_name, $name);
	if($value eq "2"){
		$value="cured";
	}elsif($value eq "1"){
		$value="remove";
	}else{
		$value="unchanged";
	}
	return $value;
}

sub get_sublist_as_str{
	my( $db_name, $name) = @_;
	my $cmd = $kldbedit." --vc=".$name." ".$db_name;
	my $value = "";
	open2(*OUTPUT, *INPUT, $cmd);
	my $first_line = <OUTPUT>;
	while( $first_line  = <OUTPUT> )
	{
   		if ($first_line=~/^[ \t]*?Name: (.*?) Value: .*?( | Subtopics:.*)$/)
		{
			my $full_name = $1;
			$full_name =~ s/^$name\///;
			if( !$value eq "" ){
				$value="$value, $full_name";
			}
			else{
				$value="$full_name";
			}
		}	
		else
		{
			print("Could not get value \"$name\" from db \"$db_name\".\n");
		}
	}
	close(INPUT);
	close(OUTPUT);
	return $value;
}

sub get_sublist_as_hash{
	my( $db_name, $name, $rm_name) = @_;
	my $cmd = $kldbedit." --vc=".$name." ".$db_name;
	my %value;
	open2(*OUTPUT, *INPUT, $cmd);
	my $first_line = <OUTPUT>;
	while( $first_line  = <OUTPUT> )
	{
   		if ($first_line=~/^[ \t]*?Name: (.*?) Value: .*?( | Subtopics:.*)$/)
		{
			my $full_name = $1;
			if( $rm_name==1 ){
				$full_name =~ s/^$name\///;
			}
			$value{$full_name}="";
		}	
		else
		{
			print("Could not get value \"$name\" from db \"$db_name\".\n");
		}
	}
	close(INPUT);
	close(OUTPUT);
	return %value;
}

my %users = get_sublist_as_hash($users_db_name, "Users_root", 1);
my $user;
open (USERS_FILE,">$users_file") or die "Could not open file with users list: $!";
foreach $user (keys (%users)){
    print USERS_FILE $user."\n";
}
close USERS_FILE;

$keeper_sections{"path"}{"LocalSocketPath"}=get_setting($db_name, "GateWarden/daemonlink/linkparam");
$keeper_sections{"path"}{"TempPath"}=get_setting($db_name, "GateWarden/directory/tmp");
$keeper_sections{"path"}{"UserFile"}=$users_file;

$keeper_sections{"$mod_name.general"}{"ForwardMailer"}=get_setting($db_name, "GateWarden/transport/mailer_admin");

$keeper_sections{"$mod_name.report"}{"ReportOk"}=get_notbool_setting($db_name, "GateWarden/log/logonlyinfected");
$keeper_sections{"$mod_name.report"}{"ReportFile"}=get_setting($db_name, "GateWarden/log/logfile");

$keeper_sections{"$mod_name.limits"}{"MaxRecipient"}=get_setting($db_name, "GateWarden/limits/maxrcpt");

$keeper_sections{"$mod_name.notify"}{"NotifyFromAddress"}=get_setting($db_name, "GateWarden/daemonlink/daemonname");
$keeper_sections{"$mod_name.notify"}{"NotSendNotifyTo"}=get_sublist_as_str($db_name, "GateWarden/limits/not_send_notify");

print <<NEWCONFIG

[path]
LicensePath=$sections{'Configuration'}{'KeysPath'}
BasesPath=$sections{'Configuration'}{'BasePath'}
LocalSocketPath=$keeper_sections{'path'}{'LocalSocketPath'}
TempPath=$keeper_sections{'path'}{'TempPath'}
UserFile=$keeper_sections{'path'}{'UserFile'}

# Scanner configuration

[scanner.options]
ExcludeDir=$sections{'Configuration'}{'ExcludeDir'}
ExcludeMask=$sections{'Configuration'}{'ExcludeMask'}
Packed=$sections{'Object'}{'Packed'}
Archives=$sections{'Object'}{'Archives'}
SelfExtArchives=$sections{'Object'}{'SelfExtArchives'}
MailBases=$sections{'Object'}{'MailBases'}
MailPlain=$sections{'Object'}{'MailPlain'}
Heuristic=$sections{'Object'}{'CodeAnalyser'}
LocalFS=$sections{'Object'}{'CrossFS'}
Cure=$sections{'Object'}{'InfectedAction'}
Recursion=$sections{'Object'}{'SubDirectories'}

#New param 
[scanner.object]
OnInfected=
OnSuspicion=
OnWarning=
OnCorrupted=

#New param
[scanner.container]
OnInfected=
OnSuspicion=
OnWarning=
OnCorrupted=

[scanner.report]
ReportFileName=$sections{'Report'}{'ReportFileName'}
#New param
ReportLevel=
#New param
ReportDetails=
Append=$sections{'Report'}{'Append'}
ShowOk=$sections{'Report'}{'ShowOk'}
#New param
ShowInfected=
ShowSuspicions=$sections{'Report'}{'ShowSuspision'}
ShowWarnings=$sections{'Report'}{'ShowWarning'}
ShowCorrupted=$sections{'Report'}{'ShowCorrupted'}
#New param
ShowCured=
#New param
ShowCureFailed=
ShowProtected=$sections{'Report'}{'ShowPassworded'}
#New param
ShowErrors=
[kavscanner.display]
ShowOk=
ShowArchiveContent=
ShowProgress=

# Daemon configuration

[aveserver.options]
#new param
DetachFromTerminal=1
#new param
StartupMode=normal
#new param
LocalSocketPermission=666

[aveserver.report]
Append=$sections{'Report'}{'Append'}
ReportFileName=$sections{'Report'}{'ReportFileName'}
#new param
ReportLevel=
#new param
ScanReportFormat=

# Keeper configuration

[$mod_name.general]
ForwardMailer=$keeper_sections{$mod_name.'.general'}{'ForwardMailer'}

#New param 
[$mod_name.license]
LicenseType=
LicenseWarningNotifyAddress =
LicenseWarningNotifySize=

[$mod_name.report]
ReportOk=$keeper_sections{$mod_name.'.report'}{'ReportOk'}
ReportFile=$keeper_sections{$mod_name.'.report'}{'ReportFile'}
#New param
ReportFilePermission=

[$mod_name.limits]
MaxRecipient=$keeper_sections{$mod_name.'.limits'}{'MaxRecipient'}
#New param
MaxCheckTime=

[$mod_name.notify]
NotifyFromAddress=$keeper_sections{$mod_name.'.notify'}{'NotifyFromAddress'}
NotSendNotifyTo=$keeper_sections{$mod_name.'.notify'}{'NotSendNotifyTo'}

NEWCONFIG
;


my %groups = get_sublist_as_hash($db_name, "GateWarden/Groups", 1);
my %msg_types;
$msg_types{"Infected"}="infected";
$msg_types{"Suspicious"}="suspicouse";
$msg_types{"Cured"}="cured";
$msg_types{"Corrupted"}="corrupted";
$msg_types{"Filtered"}="filtered";
$msg_types{"Corrupted"}="corrupted";
my $grp_name;
foreach $grp_name (keys (%groups)){
	$keeper_sections{"$mod_name.group:$grp_name"}{"Check"}=get_bool_setting($db_name, "GateWarden/Groups/$grp_name/check");
	$keeper_sections{"$mod_name.group:$grp_name"}{"Senders"}=get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/from");
	$keeper_sections{"$mod_name.group:$grp_name"}{"Recipints"}=get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/to");
	$keeper_sections{"$mod_name.group:$grp_name"}{"FilterByName"}=get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/filter");
	$keeper_sections{"$mod_name.group:$grp_name"}{"FilterByMIME"}=get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/filter_mime");
	$keeper_sections{"$mod_name.group:$grp_name"}{"FilterNotCheckName"}=get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/not_check");
	$keeper_sections{"$mod_name.group:$grp_name"}{"FilterNotCheckMIME"}=get_sublist_as_str($db_name, "GateWarden/Groups/$grp_name/not_check_mime");
#---------------------------------------------------------------------------------------------------
	print <<NEWCONFIG
[$mod_name.group:$grp_name]
Check=$keeper_sections{$mod_name.'.group:'.$grp_name}{'Check'}
Senders=$keeper_sections{$mod_name.'.group:'.$grp_name}{'Senders'}
Recipints=$keeper_sections{$mod_name.'.group:'.$grp_name}{'Recipints'}
FilterByName=$keeper_sections{$mod_name.'.group:'.$grp_name}{'FilterByName'}
FilterByMIME=$keeper_sections{$mod_name.'.group:'.$grp_name}{'FilterByMIME'}
FilterNotCheckName=$keeper_sections{$mod_name.'.group:'.$grp_name}{'FilterNotCheckName'}
FilterNotCheckMIME=$keeper_sections{$mod_name.'.group:'.$grp_name}{'FilterNotCheckMIME'}
NEWCONFIG
;
#---------------------------------------------------------------------------------------------------
	my $msg_type;
	foreach $msg_type (keys (%msg_types)){
		my $old_msg_type = $msg_types{$msg_type};
		$keeper_sections{"$mod_name.group:$grp_name"}{$msg_type."Quarantine"}=get_bool_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/isolator");
		$keeper_sections{"$mod_name.group:$grp_name"}{$msg_type."AdminNotify"}=get_bool_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/admin_send_notify");
		$keeper_sections{"$mod_name.group:$grp_name"}{$msg_type."AdminAction"}=get_action_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/admin_action");
		$keeper_sections{"$mod_name.group:$grp_name"}{$msg_type."SenderNotify"}=get_bool_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/sender_send_notify");
		$keeper_sections{"$mod_name.group:$grp_name"}{$msg_type."RecipientNotify"}=get_bool_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/user_send_notify");
		$keeper_sections{"$mod_name.group:$grp_name"}{$msg_type."RecipientAttachReport"}=get_notbool_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/user_block_mail");
		$keeper_sections{"$mod_name.group:$grp_name"}{$msg_type."RecipientAction"}=get_action_setting($db_name, "GateWarden/Groups/$grp_name/$old_msg_type/user_action");
#---------------------------------------------------------------------------------------------------
		print <<NEWCONFIG
${msg_type}Quarantine=$keeper_sections{$mod_name.'.group:'.$grp_name}{$msg_type.'Quarantine'}
${msg_type}AdminNotify=$keeper_sections{$mod_name.'.group:'.$grp_name}{$msg_type.'AdminNotify'}
${msg_type}AdminAction=$keeper_sections{$mod_name.'.group:'.$grp_name}{$msg_type.'AdminAction'}
${msg_type}SenderNotify=$keeper_sections{$mod_name.'.group:'.$grp_name}{$msg_type.'SenderNotify'}
${msg_type}RecipientNotify=$keeper_sections{$mod_name.'.group:'.$grp_name}{$msg_type.'RecipientNotify'}
${msg_type}RecipientAttachReport=$keeper_sections{$mod_name.'.group:'.$grp_name}{$msg_type.'RecipientAttachReport'}
${msg_type}RecipientAction=$keeper_sections{$mod_name.'.group:'.$grp_name}{$msg_type.'RecipientAction'}
NEWCONFIG
;
#---------------------------------------------------------------------------------------------------
	}
#---------------------------------------------------------------------------------------------------
		print <<NEWCONFIG
#New params
ErrorQuarantine=
ErrorAdminNotify=
ErrorAdminAction=
ErrorSenderNotify=
ErrorRecipientNotify=
ErrorRecipientAttachMail=
ErrorRecipientAction=
ProtectedQuarantine=
ProtectedAdminNotify=
ProtectedAdminAction=
ProtectedSenderNotify=
ProtectedRecipientNotify=
ProtectedRecipientAttachMail=
ProtectedRecipientAction=

NEWCONFIG
;
#---------------------------------------------------------------------------------------------------
}
#my %notify_types;
$keeper_sections{"$mod_name.notify.recipient"}{"Charset"}=get_setting($db_name, "GateWarden/messages/recipient_msg/charset");
$keeper_sections{"$mod_name.notify.recipient"}{"Subject"}=change_macro(get_setting($db_name, "GateWarden/messages/recipient_msg/subject"));
$keeper_sections{"$mod_name.notify.recipient"}{"ContentType"}=get_setting($db_name, "GateWarden/messages/recipient_msg/content_type");
$keeper_sections{"$mod_name.notify.recipient"}{"Content"}=change_macro(get_mlsetting($db_name, "GateWarden/messages/recipient_msg/content"));

$keeper_sections{"$mod_name.notify.sender"}{"Charset"}=get_setting($db_name, "GateWarden/messages/sender_msg/charset");
$keeper_sections{"$mod_name.notify.sender"}{"Subject"}=change_macro(get_setting($db_name, "GateWarden/messages/sender_msg/subject"));
$keeper_sections{"$mod_name.notify.sender"}{"ContentType"}=get_setting($db_name, "GateWarden/messages/sender_msg/content_type");
$keeper_sections{"$mod_name.notify.sender"}{"Content"}=change_macro(get_mlsetting($db_name, "GateWarden/messages/sender_msg/content"));

$keeper_sections{"$mod_name.notify.admin"}{"Charset"}=get_setting($db_name, "GateWarden/messages/admin_msg/charset");
$keeper_sections{"$mod_name.notify.admin"}{"Subject"}=change_macro(get_setting($db_name, "GateWarden/messages/admin_msg/subject"));
$keeper_sections{"$mod_name.notify.admin"}{"ContentType"}=get_setting($db_name, "GateWarden/messages/admin_msg/content_type");
$keeper_sections{"$mod_name.notify.admin"}{"Content"}=change_macro(get_mlsetting($db_name, "GateWarden/messages/admin_msg/content"));


#---------------------------------------------------------------------------------------------------
print <<NEWCONFIG
#New params
[$mod_name.notify]
Charset=
Subject=
ContentType=
Content=

NEWCONFIG
;
#---------------------------------------------------------------------------------------------------
my @notify_types=("infected", "suspicious", "cured", "corrupted", "filtered", "error", "protected");
my @rcpt_types=("admin", "sender", "recipient");
my $notify_type;
my $rcpt_type;
foreach $rcpt_type (@rcpt_types){
    my $template_name = $template_path.$rcpt_type."_notify_template";
    open (TEMPLATE,">$template_name") or die "Could not open template file: $!";
    print TEMPLATE $keeper_sections{$mod_name.'.notify.'.$rcpt_type}{'Content'};
    close TEMPLATE;
    foreach $notify_type (@notify_types){
#---------------------------------------------------------------------------------------------------
print <<NEWCONFIG
[$mod_name.notify.$rcpt_type.$notify_type]
Charset=$keeper_sections{$mod_name.'.notify.'.$rcpt_type}{'Charset'}
Subject=$keeper_sections{$mod_name.'.notify.'.$rcpt_type}{'Subject'}
ContentType=$keeper_sections{$mod_name.'.notify.'.$rcpt_type}{'ContentType'}
Content=$template_name

NEWCONFIG
;
#---------------------------------------------------------------------------------------------------
    }
}
