#!/usr/bin/perl 

use strict;
my $uninstall=shift;
my $source='./source';
my $version_file='./VERSION';
my $glog;
my $tmpglog;
my $productname_file='./NAME';
my $productname='';
my $product='';
my $license;
my $kavconfig='/etc/kav/5.0/kav4unix.conf';
my $hostname=`hostname`;
my $smtp_bin;
my $smtp_config;
my $license_installed = 0;
my $lic_path = "/var/db/kav/5.0/kav4unix/keys/";
my $log_path = "/var/log/kav/";
my ($bindir, $contribdir, $mandir, $confdir);

$tmpglog='/tmp/kav_install_log_.'.$$;

#open (TMPLOG,"> $tmpglog") or die "Cannot open tmp log file:$!";
#close TMPLOG;

&uninstall if $uninstall=~/uninstall/;

sub config_backup {
   my ($prod,$conf);
   return 1 unless (-f '/etc/kav/5.0/location');
   open (BC,"< /etc/kav/5.0/location") or die "Cannot open file:$!\n";
   while(my $line = <BC>) {
     if ($line =~ /NAME\s*(.*)/) {
         $prod = $1;
	 next;
     }     
     if ($line =~ /^file\t(.*\.conf)\s/ ) {
         my $old = $1 ;
	 my $new = $1.'.bak' ;
	 if (-f $old) {
	   `cp $old $new`;
	    print "\nOld config was found and copied to $new\n\n";
	 }    
         last;    
     } 
   }
   close BC;
} 

&config_backup();

sub uninstall {
   my $glog = '/etc/kav/5.0/location';
   unless (-s $glog ) {
      print "File /etc/kav/5.0/location not exists: $!\n";
   }
   open(LOG,"< $glog") or die "Cannot open installation log file.\n";
   while(my $line=<LOG>){
      if ($line =~s/^file\t(.*)/$1/) {
         $line=~s/\s+//;
	 unlink $line ;
	 print "file $line removed\n";
      } 
      if ($line =~ /(CONFDIR|CONTRIBDIR|DOCDIR|MANDIR|BINDIR)\s/) {
        $line=~s/^answer\t$1\s(.*)$/$1/;
	chomp $line;
	if (-d $line){
	   my $remove=get_answer("Would you like to remove directory $line",'no');
	   next if ($remove eq 'no');
	   `rmdir $line`;
	   print "Directory $line removed\n";
	}
      }
   }
   close LOG or die "Cannot close file.\n";
   print "Uninstall was completed.\n";
   exit 0;
}


sub perm_str2bin {
  my $perm_str = shift;
  my $ownperm = substr($perm_str,1,3);
  $ownperm =~ s/(.?)(.?)(.?)/$3$2$1/;
  my $o=0;
  for (my $i=0;$i<3;$i++) {
    $o+=2**$i if (not (substr($ownperm,$i,1) eq '-')) ;
  }   
  my $groupperm = substr($perm_str,4,3);
  $groupperm =~ s/(.?)(.?)(.?)/$3$2$1/;
  my $g=0;
  for (my $i=0;$i<3;$i++) {
    $g+=2**$i if (not (substr($groupperm,$i,1) eq '-'));
  } 
  my $otherperm = substr($perm_str,7,3);
  $otherperm =~ s/(.?)(.?)(.?)/$3$2$1/;
  my $a=0;
  for (my $i=0;$i<3;$i++) {
    $a+=2**$i if (not (substr($otherperm,$i,1) eq '-'));  
  }	
  return $o.$g.$a ;
}

sub internal_basename {
  return substr($_[0], rindex($_[0], '/') + 1);
}

sub internal_dirname {
  my $path = shift;
  my $pos;

  $path = dir_remove_trailing_slashes($path);

  $pos = rindex($path, '/');
  if ($pos == -1) {
    # No slash
    return '.';
  }

  if ($pos == 0) {
    # The only slash is at the beginning
    return '/';
  }

  return substr($path, 0, $pos);
}


sub get_install_version {
  open (VER,"<$version_file") or die "Cannot detect installation version: $! \n";
  my $version = <VER>;
  close VER;
  return $version;
}

sub get_productname {
  open (NAME,"<$productname_file") or die "Cannot detect installation product name: $! \n";
  my $productname = <NAME>;
  close NAME;
  return $productname;
}

sub error {
  my $msg = shift;

  print STDERR $msg." Execution aborted."."\n\n" ;
  exit 1;
}


sub safe_chmod {
  my $mode = shift;
  my $file = shift;

  if (chmod($mode, $file) != 1) {
    error('Unable to change the access rights of the file ' . $file . '.' . "\n\n");
  }
}


sub safe_mkdir {
  my $file = shift;

  if (mkdir($file, 0000) == 0) {
    error('Unable to create the directory ' . $file . '.' . "\n\n");
  }
}


sub create_dir {
  my $dir = shift;

  if (-d $dir) {
    return 1;
  }

  if (index($dir, '/') != -1) {
    create_dir(internal_dirname($dir));
  }
  safe_mkdir($dir);
  safe_chmod(0755, $dir);
  return 0;
}


my $ver=get_install_version ;
my $name=get_productname ;
 
sub start_install {
  my $ver=get_install_version ;
  my $name=get_productname ;
  create_dir ('/var/db/kav/5.0/kav4unix/bases');
  create_dir ($log_path);
  safe_chmod(0777,$log_path);
  create_dir ('/var/db/kav/5.0/kav4unix/bases.backup');
  open(TMPLOG,"> $tmpglog") or die "Cannot open file: $!";
  print TMPLOG "NAME ". $name ;
  print TMPLOG "VERSION ". $ver ;
  close TMPLOG ;
}


sub get_new_installated {
	my $version;
	my $patch_version;
  unless (-s $glog) {
    return 1 ;	  
  }
  open (NEWINSTALL,"< $glog") or die "Cannot open file: $!\n";
  while(<NEWINSTALL>){
    if ($_=~/^VERSION\s(.*)/) {
      $version = $1;
    }
    else {
      $version = '';
    }
    if ($_=~/^PATCH\s(.*)/) {
      $patch_version = $1;
    }
    else {
      $patch_version = '';
    }
  }
  return ($version,$patch_version);
}    	  
  
sub install_dir {
  my $src = shift;
  my $top = shift;
  my $dst = $top ;
  my $dir;
  my $name = './'.$src;
  open (SRC,"<$source") or die "Cannot open file:$!";
  while(my $line=<SRC>){
    my @a = split (/\s+/,$line);
    if (($a[0] =~ /(^\.\/.*)\:$/)&($1 eq "./".$src)) {
      $dir=$1 ;
      create_dir($top);    
    }
    if ($a[8] =~ /(\S+)/) {
      my $file=$dir."/".$1;	
      if (-f $file) {
        my $dstfile=$top;      
        `cp $file $dstfile`;	
        my $perm = perm_str2bin($a[0]);
	 $file=internal_basename($file);
        `chmod $perm $dstfile/$file`;
        `chown $a[2]:$a[3] $dstfile/$file`;
	add_to_log($tmpglog,'file',$dstfile.'/'.internal_basename($file));
      }
    }
    
  }
  close SRC;
} 

sub dir_remove_trailing_slashes {
  my $path = shift;
  for(;;) {
    my $len;
    my $pos;
    $len = length($path);
    if ($len < 2) {
    # Could be '/' or any other character. Ok.
      return $path;
    }
    $pos = rindex($path, '/');
    if ($pos != $len - 1) {
    # No trailing slash
      return $path;
    }
    # Remove the trailing slash
    $path = substr($path, 0, $len - 1)
  }
}

sub is_root {
  return $> == 0;
}

sub query {
  my $message = shift;
  my $default = shift;
  my $answer;
  print ($message . (($default eq '') ? '' : (' [' . $default . ']:')));
  chomp($answer = <STDIN>);
  if ($answer eq '') {
    return $default;  
  }    
    return $answer;
}

sub add_to_log {
  my $tmpglog = shift;
  my $type = shift;
  my $var = shift;
  my $reply = shift;
  if (not open(TMPLOG,">> $tmpglog")) {
    print "Unable to open ".$tmpglog." in append mode.\n\n";
    
  }
  print TMPLOG $type."	".$var." ".$reply."\n";
  close TMPLOG;
}

sub get_answer {
  my $message = shift;
  my $default = shift;
  my $type = shift;
  my $var = shift;
  my $answer;
  $answer = query($message,$default);
  add_to_log($tmpglog,$type,$var,$answer);
  return $answer;
}

sub check_initscriptsdir {
  my $dir = shift;
  $dir = dir_remove_trailing_slashes($dir); 
  my $source = shift;
  if (not (-d $dir)) {
    if ($source eq 'user') {
      print "The path ".$dir." not exists\n";	    
    }	    
    return '';
  }
  return $dir;
}

sub remove_whitespaces {
  my $string = shift;

  $string =~ s/^\s*//;
  $string =~ s/\s*$//;
  return $string;
}

sub is_dir_exist{
  my $dir = shift;
  if (-d $dir) {
    return 0;
  }
  return 1;
}

################
#begin of main()
################

if (not is_root()) {
  print "You must run this program as root.\n";	
  exit 1;
}



# Search previos version Kaspersky AV
# If it founded to suggest backup it and remove it.
my $name;
my %packages;
sub get_old_installated {
  unless (-s '/etc/install.avp') {
    return 1;
  }
  open (OLDINSTALL,'</etc/install.avp') or die "Cannot open file for reading:$!\n";
  while(my $line = <OLDINSTALL>) {
    if ($line=~/^\[(.*)\]$/) {
      $name=$1;
    }
    elsif (($line=~/^(.*)\=(.*)/) & ($line !~ /^File.*/)) {
      $packages{$name}{$1}=$2 ;
    }
    elsif ($line=~/^File\=(\S+)(\s+)(.*)/) {
      push @{$packages{$name}{'files'}},$packages{$name}{'SubDir'}.'/'.$1;
    }	  
  }
  return 0;
}

sub internal_ls {
  my $dir = shift;
  my @fn;

  opendir(LS, $dir);
  @fn = grep(!/^\.\.?$/, readdir(LS));
  closedir(LS);

  return @fn;
}


sub get_key {
  my $dir = shift ;
  my @fn ;
  opendir(LS,$dir) ;
  @fn = grep(s/(.+(\.key))$/$dir\/$1/, readdir(LS));
  closedir(LS);
  return @fn;
} 

sub internal_basename {
  return substr($_[0], rindex($_[0], '/') + 1);
}
 
sub key_install {
  my $license_path='/var/db/kav/5.0/kav4unix/keys';
  print "\n";
  create_dir($license_path);
  if (my @keys=get_key("./")) {
  	print "Installing licenses from current directory.\n";
    foreach (@keys) {
      unless (-f $license_path."/".internal_basename($_)) {
          `cp $_ $license_path`;
      }
	  $license_installed = 1;
    }
  }
  else {
    while(1) {
      my $dir=get_answer ("Kaspersky Anti-Virus requires license keys (*.key) to work.\n"
      . "If you don't have any license file, read the documentation how to get them.\n"
      . "Please enter directory which contains Kaspersky Lab license key files\n"
      . "(type \"cancel\" for continue installation without license installation.)",'cancel','answer','KEY');
      if ($dir =~ /^cancel$/i) {
        return 1;
	last;
      }
      if (my @keys=get_key($dir)) {
        foreach (@keys) {
          unless (-f $license_path."/".internal_basename($_)) {
            `cp $_ $license_path`;
            print "Installing license file " . internal_basename($_) . "\n";
	    $license_installed = 1;
          }
          else {
            print "You have already installed the license file ".internal_basename($_)."\n";
          }
        }
        last;
      }
      else {
        print "License file not found.\n";
      }
    }
  }    
}


sub backup {
  my $archivename = get_answer("Where do you want to save backup file.
  Enter absolute path to file with out .tgz extension","/tmp/oldkav",'answer','BACKUP_PATH');	
  my $file = $archivename.'.tgz';
  while (-f $file ){
    if (get_answer("File ".$file." exists.Do you want overwrite it?","yes",'answer','BACKUP_OVERWRITE') eq 'yes') {	  
     last ; 
    } 
    else {
        $archivename = get_answer("Where do you want to save backup file.
  Enter absolute path to file with out .tgz extension","/tmp/oldkav",'answer','BACKUP_PATH');	  
        $file = $archivename.'.tgz';
        next ;
    }
  }    
  unless (-d internal_dirname($file)){
    create_dir(internal_dirname($file));    
  }
  foreach $name (keys %packages) { 
    if (system("tar -czvf $file @{$packages{$name}{'files'}} 2>/dev/null") == 1){    
      print "Backup completed\n\n";
    } 
    else {
      print "Backup not completed\n\n"; 
    }
  }       
}

sub convert {
  my $tmpconfig = shift;  
  `contrib/converter.pl > $tmpconfig` ;
  if(-f $tmpconfig) {
    if(get_answer("Do you want to make new generated config file default?",'yes','answer','CONFIG_CONVVERT') eq 'yes') {
      `mv /etc/kav/5.0/kav4unix.conf /etc/kav/5.0/kav4unix.conf.origin`;
      `mv $tmpconfig /etc/kav/5.0/kav4unix.conf`;
      
    }
    else {
      print "New generated file will be to store as ".$tmpconfig."\n";
      
    }
  }    
}




# Stoped all Kavspersky programm
# Inform about result.

if (get_old_installated() == 0) {
  if (get_answer("Previously installed components of Kaspersky Anti-Virus are found.\nDo you want to back then up now?","yes",'answer','BACKUP') eq 'yes') {
    backup();
  }
  if (get_answer("Do you want to convert old settings to a new config file?","yes",'answer','CONVERT') eq 'yes') {
    convert('/etc/kav/5.0/kav4mailserver.conf_4.0');
  }
  if (get_answer("Do you want to remove old files?","yes",'answer','REMOVE_OLD') eq 'yes') {
    foreach $name (keys %packages) {
      unlink @{$packages{$name}{'files'}} ;
      rmdir '/opt/AVP' ;
      unlink '/etc/install.avp' ;
    }
  }
}


print "Installing the package\n";
start_install();
# For install configuration files
while ( 1 ) {
	$confdir = get_answer ("Which directory do you want to install the configuration files in?", "/etc/kav/". substr (get_install_version,0,3), 'answer', 'CONFDIR');
	last unless is_dir_exist($confdir);
	last if (get_answer ("The path $confdir does not exist. Would you like to create it?", "yes", 'answer', 'CONFDIR_CREATE') eq 'yes');
}

install_dir ('etc', $confdir);


$glog=$confdir.'/location';




# For install binary files

while ( 1 ) {
	$bindir = get_answer ("Which directory do you want to install the binary files in?", "/opt/kav/bin", 'answer', 'BINDIR');
	last unless is_dir_exist($bindir); 
	last if (get_answer ("The path $bindir does not exist. Would you like to create it?", "yes", 'answer', 'BINDIR_CREATE') eq 'yes');
}

install_dir ('bin', $bindir);

# For install contrib files
while ( 1 ) {
	$contribdir = get_answer ("Which directory do you want to install the contrib files in?", "/opt/kav/contrib", 'answer', 'CONTRIBDIR');
	last unless is_dir_exist($contribdir);
	last if (get_answer ("The path $contribdir does not exist. Would you like to create it?", "yes", 'answer', 'DOCDIR_CREATE') eq 'yes');
}

install_dir ('contrib', $contribdir);

# For install man files
while ( 1 ) {
	$mandir = get_answer ("Which directory do you want to install the manual files in?", "/opt/kav/man", 'answer', 'MANDIR');
	last unless is_dir_exist($mandir) ; 
	last if (get_answer ("The path $mandir does not exist. Would you like to create it?", "yes", 'answer', 'MANDIR_CREATE') eq 'yes');
}

install_dir ('man', $mandir);

#key_install();

# For webmin install
sub webmin_module_install {
  my $path_script;
  my $webmin_conf="/etc/webmin"; 
  while (is_dir_exist($webmin_conf)) {
    $webmin_conf=get_answer("\nWebmin configuration file is not found.\n"
    	. "(probably webmin is not installed, or installed into non-default location.)\n\n"
    	. "Webmin (www.webmin.com) is a web-based interface for system administration\n"
    	. "for various Unix utilites. If you install it, you'll be able to configure\n"
    	. "and use Kaspersky Anti-Virus through the web interface.\n"
    	. "If you want to use this functionality, but haven't installed webmin yet,\n"
    	. "you can safely type 'cancel' here, install webmin later and re-run this\n"
    	. "configuration script again (or use Webmin to install the module from directory\n"
    	. "$contribdir/kavfs.wbm)\n"
    	. "If you have webmin installed in non-default path, please enter\n"
    	. "where webmin configuration file located?\n(type \"cancel\" for continue installation without webmin)." ,
     		'cancel') ;
    if($webmin_conf eq 'cancel') {
      return 1;
    }
  }

  unless ($webmin_conf eq 'cancel') {
    open (WBM,"<$webmin_conf/miniserv.conf") or (warn "Could not open config webmin file. Webmin module installation aborted." and return 1);
    while(<WBM>) {
      $path_script=$1 if (/^root=(.*)$/);
     }       
    close WBM;
    `perl $path_script/install-module.pl $contribdir/kavfs.wbm $webmin_conf` ;
    print "Webmin module installed successfully. Check it on 'Others' web page.\n"
    	. "Depend on Webmin configuration, you may need to enable Kaspersky Anti-Virus\n"
    	. "Webmin module. To do this, on Webmin main page go to\n" 
    	. "Webmin -> Webmin users -> <username> -> check box on 'KAV for Mail Servers'\n";

  }
}

webmin_module_install();

# For configuration

sub useradd {
   my $user = shift;
   my $group = shift; 
   my $home = shift;
   my $shell = shift; 
   unless (-d $home) {
      system("mkdir -p $home") == 0 or warn "Cannot create directory:$!\n"; 
      return 1; 
   }
   unless(getpwnam($user)){
      system("groupadd $group") == 0 or warn "Cannot create group filter:$!\n";
      system("useradd $user -d $home -s $shell  -g $group") == 0 or warn "Cannot create user filter:$!\n";  
   }
   my ($login,$pass,$uid,$gid) = getpwnam($user)
                       or die "User $user not in passwd file";
   chown($uid, $gid, $home) ; 
}


#For license
my $license;
my $lic_path;
open (KAVCONF,"+< $kavconfig") or die "Cannot open config file\n";
  my @KAVCONFIG=<KAVCONF> ;
  foreach my $param (@KAVCONFIG) {
    if ($param=~/^LicensePath=(.*)/) {
         $lic_path=$1; 
      }
  }
close KAVCONF or die "Closing :$!\n";

&key_install($lic_path);

rename "confdir/users",$lic_path ;

if (my @keys=get_key($lic_path)) {
  foreach (@keys) {   
    my $key='/var/db/kav/5.0/kav4unix/keys'."/".internal_basename($_);
    if (-f $key ) { 
       my $lic_type =  $? >> 8 if(system("/opt/kav/bin/licenseviewer -k $key > /dev/null")) ;
       $license = 'users' if $lic_type == 1;
       $license = 'type' if $lic_type == 2;
    }
  }
  open (KAVCONF,"+< $kavconfig") or die "Cannot open config file\n";
  my @KAVCONFIG=<KAVCONF> ;
  foreach my $param (@KAVCONFIG) {
    $param=~s/^LicenseType=(.*)/LicenseType=$license/ ;
  }
  seek(KAVCONF,0,0) or die "Seeking :$!\n";
  print KAVCONF @KAVCONFIG or die "Printing :$!\n";
  truncate(KAVCONF,tell(KAVCONF)) or die "Truncating :$!\n";
  close KAVCONF;
}

`mv $tmpglog $glog` ;
 
print "******************************************************************************\n"
	. "Kaspersky Anti-Virus for ".uc($^O)." configuration completed.\n"
	. "Binaries were installed in $bindir directory\n"
	. "Configuration file is $kavconfig\n"
	. "We recommend to add this directory in your PATH environment variable.\n"
	. "To access the man pages add the /opt/kav/man path to the MANPATH env variable\n"
	. "To start using Kaspersky Anti-Virus you should download latest AV bases;\n"
	. "to do this run $bindir/keepup2date.\n"
	. "Also if you're behind the proxy and want to update from Internet, you should\n"
	. "set environment variables http_proxy and/or ftp_proxy (see wget manual pages)\n"
	. "To enable mail server protection, restart your mail system as written in\n"
	. " product documentation. \n";

unless ( $license_installed )
{
	print "\nWARNING: You didn't install any license. Before using the product, you must\n"
		. "install valid licenses (just copy them to the directory specified in\n"
		. "LicensePath - currently '$lic_path').\n"
		. "If you need to obtain the license, see the product documentation.\n"
}	
